package net.wanji.web.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.vo.*;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.dto.*;
import net.wanji.web.mapper.scheme.*;
import net.wanji.web.po.scheme.*;
import net.wanji.web.service.FeignProxyService;
import net.wanji.web.service.PlanSendService;
import net.wanji.web.service.scheme.impl.RunningPlanServiceImpl;
import net.wanji.web.service.scheme.impl.SchemeConfigServiceImpl;
import net.wanji.web.vo.PhaseListByTimeVO;
import net.wanji.web.vo.SchemePhaseLightsVO;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/2/1 9:39
 */
@Service
public class PlanSendServiceImpl implements PlanSendService {
    private final UtcFeignClients utcFeignClients;
    private final FeignProxyService feignProxyService;
    private final CrossSchedulesPlanMapper crossSchedulesPlanMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final RunningPlanServiceImpl runningPlanServiceImpl;
    private final SchemeConfigServiceImpl schemeConfigServiceImpl;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final CrossLaneLightsMapper crossLaneLightsMapper;
    private final CrossSectionMapper crossSectionMapper;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    SimpleDateFormat hourMinuteSdf = new SimpleDateFormat("HH:mm");

    private Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd").create();

    public PlanSendServiceImpl(@Qualifier("net.wanji.feign.service.UtcFeignClients") UtcFeignClients utcFeignClients,
                               CrossSchedulesPlanMapper crossSchedulesPlanMapper, CrossPlanMapper crossPlanMapper,
                               CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper,
                               CrossLightsMapper crossLightsMapper, CrossPhaseLightsMapper crossPhaseLightsMapper,
                               RunningPlanServiceImpl runningPlanServiceImpl, SchemeConfigServiceImpl schemeConfigServiceImpl,
                               CrossSchedulesMapper crossSchedulesMapper, CrossLaneLightsMapper crossLaneLightsMapper,
                               CrossSectionMapper crossSectionMapper, FeignProxyService feignProxyService) {
        this.utcFeignClients = utcFeignClients;
        this.crossSchedulesPlanMapper = crossSchedulesPlanMapper;
        this.crossPlanMapper = crossPlanMapper;
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
        this.crossLightsMapper = crossLightsMapper;
        this.crossPhaseLightsMapper = crossPhaseLightsMapper;
        this.runningPlanServiceImpl = runningPlanServiceImpl;
        this.schemeConfigServiceImpl = schemeConfigServiceImpl;
        this.crossSchedulesMapper = crossSchedulesMapper;
        this.crossLaneLightsMapper = crossLaneLightsMapper;
        this.crossSectionMapper = crossSectionMapper;
        this.feignProxyService = feignProxyService;
    }

    /**
     * 相位车道参数下发
     *
     * @param crossId
     * @return
     */
    public JsonViewObject phaseLaneSend(String crossId) {
        PhaseTimingSendVO phaseTimingSendVO = buiPhaseTimingSendVO(crossId);
        JsonViewObject jsonViewObject = feignProxyService.phaseTimingSend(phaseTimingSendVO);
        return jsonViewObject;
    }

    /**
     * 构建下发相位参数
     *
     * @param crossId
     * @return
     */
    private PhaseTimingSendVO buiPhaseTimingSendVO(String crossId) {
        PhaseTimingSendVO phaseTimingSendVO = new PhaseTimingSendVO();
        phaseTimingSendVO.setCrossCode(crossId);
        List<PhaseTimingSendVO.Phase> phaseList = new ArrayList<>();
        CrossPhasePO crossPhasePO = new CrossPhasePO();
        crossPhasePO.setCrossId(crossId);
        List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.listCrossPhasePO(crossPhasePO);
        CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
        crossPhaseLightsPO.setCrossId(crossId);
        List<CrossPhaseLightsPO> crossPhaseLightsPOS = crossPhaseLightsMapper.listCrossPhaseLightsPO(crossPhaseLightsPO);
        CrossLightsPO crossLightsPO = new CrossLightsPO();
        crossLightsPO.setCrossId(crossId);
        List<CrossLightsPO> crossLightsPOS = crossLightsMapper.listCrossLightsPO(crossLightsPO);
        for (CrossPhasePO phasePO : crossPhasePOS) {
            PhaseTimingSendVO.Phase phase = new PhaseTimingSendVO.Phase();
            phase.setPhaseNo(Integer.valueOf(phasePO.getPhaseNo()));
            phase.setDesc(phasePO.getName());
            phase.setYellow(phasePO.getYellowTime());
            phase.setAllred(phasePO.getRedTime());
            phase.setGreenFlash(String.valueOf(phasePO.getGreenFlashTime()));
            phase.setRedFlash(String.valueOf(phasePO.getRedFlashTime()));
            phase.setMaxGreen(String.valueOf(phasePO.getMaxGreenTime()));
            phase.setMinGreen(String.valueOf(phasePO.getMinGreenTime()));
            Integer id = phasePO.getId();
            List<PhaseTimingSendVO.Phase.Lane> lanes = new ArrayList<>();
            for (CrossPhaseLightsPO phaseLightsPO : crossPhaseLightsPOS) {
                if (Objects.equals(id, phaseLightsPO.getPhaseId())) {
                    Integer lightsId = phaseLightsPO.getLightsId();
                    for (CrossLightsPO lightsPO : crossLightsPOS) {
                        if (Objects.equals(lightsId, lightsPO.getId())) {
                            PhaseTimingSendVO.Phase.Lane lane = new PhaseTimingSendVO.Phase.Lane();
//                            lane.setLaneNo(Integer.valueOf(lightsPO.getLightsNo()));
                            lane.setDirection(lightsPO.getDir());
                            lane.setTurn(lightsPO.getType());
                            lanes.add(lane);
                        }
                    }
                }
            }
            phase.setLanes(lanes);
            phaseList.add(phase);
        }
        phaseTimingSendVO.setPhaseList(phaseList);

        return phaseTimingSendVO;
    }

    @Override
    @Transactional
    public JsonViewObject scheduleSend(ScheduleIdDTO scheduleIdDTO) {
        String crossId = scheduleIdDTO.getCrossId();
        Integer scheduleId = scheduleIdDTO.getScheduleId();
        ScheduleSendVO scheduleSendVO = new ScheduleSendVO();
        scheduleSendVO.setCrossCode(crossId);

        // 下发调度
        List<ScheduleSendVO.Schedule> schedules = buildSchedules(crossId, scheduleIdDTO);
        scheduleSendVO.setSchedules(schedules);
        JsonViewObject jsonViewObject = utcFeignClients.scheduleSend(scheduleSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-运行计划下发UTC服务调用异常");
        }
        // 下发日计划和方案
        PlanSendVO planSendVO = getPlanSendVO(crossId, scheduleId);
        JsonViewObject planSendResult = utcFeignClients.planSend(planSendVO);
        if (Objects.isNull(planSendResult) || planSendResult.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-日计划下发UTC服务调用异常");
        }
        // 下发车道相位
        phaseLaneSend(crossId);

        // 更新时间表状态为已执行
        crossSchedulesMapper.resetStatus(crossId);
        crossSchedulesMapper.updateStatus(scheduleId);
        return jsonViewObject.success();
    }

    private PlanSendVO getPlanSendVO(String crossId, Integer scheduleId) {
        List<CrossSchedulesPlanPO> crossSchedulesPlanPOList =
                crossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, scheduleId);
        Integer planId = crossSchedulesPlanPOList.get(0).getPlanId();
        CrossPlanPO crossPlanPO = crossPlanMapper.selectById(planId);

        CrossSectionPO crossSectionPO = new CrossSectionPO();
        crossSectionPO.setCrossId(crossId);
        crossSectionPO.setPlanId(planId);
        List<CrossSectionPO> crossSectionPOS = crossSectionMapper.listCrossSectionPO(crossSectionPO);
        List<PlanSendVO.Plan.Section> sectionList = new ArrayList<>(crossSectionPOS.size());
        for (CrossSectionPO sectionPO : crossSectionPOS) {
            Integer schemeId = sectionPO.getSchemeId();
            CrossSchemePO crossSchemePO = crossSchemeMapper.selectById(schemeId);
            String patternNo = crossSchemePO.getSchemeNo();
            PlanSendVO.Plan.Section section = new PlanSendVO.Plan.Section();
            section.setSectionNo(sectionPO.getSectionNo());
            section.setPatternNo(patternNo);
            section.setBeginTime(sectionPO.getStartTime());
            section.setEndTime(sectionPO.getEndTime());
            section.setControlMode(String.valueOf(sectionPO.getControlMode()));
            sectionList.add(section);

            // 下发日计划中方案
            SchemeIdDTO schemeIdDTO = new SchemeIdDTO();
            schemeIdDTO.setCrossId(crossId);
            schemeIdDTO.setSchemeId(schemeId);
            schemeSend(schemeIdDTO);
        }

        PlanSendVO planSendVO = new PlanSendVO();
        planSendVO.setCrossCode(crossId);
        PlanSendVO.Plan plan = new PlanSendVO.Plan();
        plan.setPlanNo(crossPlanPO.getPlanNo());
        plan.setPlanDescribe(crossPlanPO.getName());
        plan.setSectionList(sectionList);
        planSendVO.setPlanList(Arrays.asList(plan));
        return planSendVO;
    }

    @Override
    public JsonViewObject schemeSend(SchemeIdDTO schemeIdDTO) {
        String crossId = schemeIdDTO.getCrossId();
        SchemeSendVO schemeSendVO = new SchemeSendVO();
        schemeSendVO.setCrossCode(crossId);
        // 构造方案列表
        Integer schemeId = schemeIdDTO.getSchemeId();
        List<SchemeSendVO.Pattern> patternList = buildPatternList(crossId, schemeId);
        schemeSendVO.setPatternList(patternList);
        // 发送请求
        JsonViewObject jsonViewObject = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-基础方案下发UTC服务调用异常");
        }
        // 下发车道相位
        phaseLaneSend(crossId);
        // 更新方案表状态为已执行
        crossSchemeMapper.resetStatus(crossId);
        crossSchemeMapper.updateStatus(schemeId);
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject nextStep(CrossIdDTO crossIdDTO) {
        String crossId = crossIdDTO.getCrossId();
        // command 1开始步进，stepNum 0顺序步进
        JsonViewObject jsonViewObject = utcFeignClients.stepControl(crossId, 1, 0);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-步进控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject lockPhase(CrossIdAndPhaseIdsDTO crossIdAndPhaseIdsDTO) {
        LockControlVO lockControlVO = new LockControlVO();
        lockControlVO.setCrossCode(crossIdAndPhaseIdsDTO.getCrossId());
        lockControlVO.setCommand(1); // 控制标识：1是；0否
        lockControlVO.setDuration(0); // 长期

        List<Integer> phaseIds = crossIdAndPhaseIdsDTO.getPhaseIds();
        List<Integer> phaseList = new ArrayList<>();
        for (Integer phaseId : phaseIds) {
            CrossPhasePO crossPhasePO = crossPhaseMapper.selectById(phaseId);
            String phaseNo = crossPhasePO.getPhaseNo();
            phaseList.add(Integer.parseInt(phaseNo));
        }
        lockControlVO.setPhaseList(phaseList);

        JsonViewObject jsonViewObject = utcFeignClients.lockControl(lockControlVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-锁定控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject allRedControl(CrossIdDTO crossIdDTO) {
        ControlCommandVO commandVO = getControlCommandVO(crossIdDTO);
        commandVO.setControlType(1); //1全红 2黄闪 3关灯
        JsonViewObject jsonViewObject = utcFeignClients.allRedControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-全红控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject yellowLightControl(CrossIdDTO crossIdDTO) {
        ControlCommandVO commandVO = getControlCommandVO(crossIdDTO);
        commandVO.setControlType(2); //1全红 2黄闪 3关灯
        JsonViewObject jsonViewObject = utcFeignClients.yellowLightControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-黄闪控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    public JsonViewObject restore(CrossIdAndPhaseIdsDTO crossIdAndPhaseIdsDTO) {
        String crossId = crossIdAndPhaseIdsDTO.getCrossId();
        List<Integer> phaseIds = crossIdAndPhaseIdsDTO.getPhaseIds();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        // 取消步进
        // command 1开始步进，stepNum 0顺序步进
        jsonViewObject = utcFeignClients.stepControl(crossId, 0, 0);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-步进控制UTC服务调用异常");
        }

        // 取消相位锁定
        LockControlVO lockControlVO = new LockControlVO();
        lockControlVO.setCrossCode(crossIdAndPhaseIdsDTO.getCrossId());
        lockControlVO.setCommand(0); // 控制标识：1是；0否
        lockControlVO.setDuration(0); // 长期
        List<Integer> phaseList = new ArrayList<>();
        for (Integer phaseId : phaseIds) {
            CrossPhasePO crossPhasePO = crossPhaseMapper.selectById(phaseId);
            String phaseNo = crossPhasePO.getPhaseNo();
            phaseList.add(Integer.parseInt(phaseNo));
        }
        lockControlVO.setPhaseList(phaseList);

        jsonViewObject = utcFeignClients.lockControl(lockControlVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-锁定控制UTC服务调用异常");
        }

        // 取消全红控制
        CrossIdDTO crossIdDTO = new CrossIdDTO();
        crossIdDTO.setCrossId(crossId);
        ControlCommandVO commandVO = getControlCommandVO(crossIdDTO);
        commandVO.setCommand(0); //控制标识：1是；0否
        commandVO.setControlType(1); //1全红 2黄闪 3关灯
        jsonViewObject = utcFeignClients.allRedControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-全红控制UTC服务调用异常");
        }

        // 取消黄闪控制
        commandVO.setControlType(2); //1全红 2黄闪 3关灯
        jsonViewObject = utcFeignClients.yellowLightControl(commandVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-黄闪控制UTC服务调用异常");
        }
        return jsonViewObject.success();
    }

    @Override
    @Transactional
    public JsonViewObject syncScheme(CrossIdDTO crossIdDTO) {
        CrossIdVO crossIdVO = new CrossIdVO();
        String crossId = crossIdDTO.getCrossId();
        crossIdVO.setCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClients.schemePhaseLights(crossIdVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机静态信息获取-方案信息-UTC服务调用异常");
        }
        Object content = jsonViewObject.getContent();
        if (ObjectUtil.isNotNull(content)) {
            // 信息入库
            String contentStr = gson.toJson(content);
            SchemePhaseLightsVO schemePhaseLightsVO = gson.fromJson(contentStr, SchemePhaseLightsVO.class);
            if (ObjectUtil.isEmpty(schemePhaseLightsVO)) {
                return jsonViewObject.fail("信号机方案对象转换结果为空，未更新数据库");
            }
            List<CrossSchemePO> crossSchemeList = schemePhaseLightsVO.getCrossSchemeList();
            crossSchemeMapper.deleteByCrossId(crossId);
            // 保存旧方案ID
            List<Integer> oldSchemeIds = crossSchemeList.stream().map(CrossSchemePO::getId).collect(Collectors.toList());
            // 保存新方案ID
            crossSchemeMapper.insertBatch(crossSchemeList);
            List<Integer> newSchemeIds = crossSchemeList.stream().map(CrossSchemePO::getId).collect(Collectors.toList());
            // 生成新旧方案ID映射
            Map<Integer, Integer> oldNewSchemeId = new HashMap<>();
            for (int i = 0; i < oldSchemeIds.size(); i++) {
                oldNewSchemeId.put(oldSchemeIds.get(i), newSchemeIds.get(i));
            }

            List<CrossPhasePO> crossPhaseList = schemePhaseLightsVO.getCrossPhaseList();
            crossPhaseMapper.deleteByCrossId(crossId);
            // 更新方案ID
            for (CrossPhasePO crossPhasePO : crossPhaseList) {
                crossPhasePO.setPlanId(oldNewSchemeId.get(crossPhasePO.getPlanId()));
            }
            crossPhaseMapper.insertBatch(crossPhaseList);

            List<CrossLightsPO> crossLightsList = schemePhaseLightsVO.getCrossLightsList();
            // 清理数据库
            crossLightsMapper.deleteByCrossId(crossId);
            // 灯组号去重
            TreeSet<CrossLightsPO> crossLightsSet = distinctLightsNo(crossLightsList);
            // 修改灯组数据内容
            changeLightsType(crossLightsSet);
            crossLightsMapper.insertBatch(crossLightsSet);

            crossPhaseLightsMapper.deleteByCrossId(crossId);
            crossLaneLightsMapper.deleteByCrossId(crossId);
        }
        return jsonViewObject.success();
    }

    private TreeSet<CrossLightsPO> distinctLightsNo(List<CrossLightsPO> crossLightsList) {
        TreeSet<CrossLightsPO> crossLightsSet = crossLightsList.stream()
                .collect(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(CrossLightsPO::getLightsNo))));
        return crossLightsSet;
    }

    private static void changeLightsType(Set<CrossLightsPO> crossLightsSet) {
        for (CrossLightsPO crossLightsPO : crossLightsSet) {
            Integer type = crossLightsPO.getType();
            if (type == 2) {
                crossLightsPO.setType(1);
            } else if (type == 1) {
                int turn = crossLightsPO.getTurn();
                if (turn == 1) {
                    crossLightsPO.setType(2);
                } else if (turn == 2) {
                    crossLightsPO.setType(3);
                } else if (turn == 3) {
                    crossLightsPO.setType(4);
                } else if (turn == 4) {
                    crossLightsPO.setType(5);
                }
            } else if (type == 3) {
                Integer inOutType = crossLightsPO.getInOutType();
                if (inOutType == 3) {
                    crossLightsPO.setType(20);
                } else if (inOutType == 1) {
                    crossLightsPO.setType(21);
                } else if (inOutType == 2) {
                    crossLightsPO.setType(22);
                }
            }
        }
    }

    @Override
    public PhaseListByTimeVO phaseListByTime(CrossIdAndTimeDTO crossIdAndTimeDTO) {
        PhaseListByTimeVO phaseListByTimeVO = new PhaseListByTimeVO();
        String crossId = crossIdAndTimeDTO.getCrossId();
        Date datetime = crossIdAndTimeDTO.getDatetime();
        String dateStr = sdf.format(datetime);
        CrossIdDTO crossIdDTO = new CrossIdDTO();
        crossIdDTO.setCrossId(crossId);
        // 获取所有计划
        RunningPlanDTO runningPlanDTO = runningPlanServiceImpl.listRunningPlan(crossIdDTO);
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            List<RunningPlanDTO.ExecListElement> execList = schedulesPlan.getExecList();
            for (RunningPlanDTO.ExecListElement execListElement : execList) {
                // 先查询特殊日期，再查询星期
                List<String> specialDateList = execListElement.getSpecialDateList();
                List<Integer> week = execListElement.getWeek();
                if (CollectionUtil.isNotEmpty(specialDateList)) {
                    for (String specialDate : specialDateList) {
                        String substring = specialDate.substring(0, 10);
                        if (Objects.equals(dateStr, substring)) {
                            String planName = execListElement.getPlanName();
                            phaseListByTimeVO = getPhaseListByTimeVO(planName, datetime, runningPlanDTO, crossIdDTO);
                        }
                    }
                } else if (CollectionUtil.isNotEmpty(week)) {
                    for (Integer day : week) {
                        // 1表示周日，2表示周一
                        int today = DateUtil.dayOfWeek(datetime);
                        today -= 1;
                        if (today == 0) {
                            today = 7;
                        }
                        if (day == today) {
                            String planName = execListElement.getPlanName();
                            phaseListByTimeVO = getPhaseListByTimeVO(planName, datetime, runningPlanDTO, crossIdDTO);
                        }
                    }
                }
            }
        }
        return phaseListByTimeVO;
    }

    @Override
    public List<SaveSchemeConfigDTO.PhaseListElement> phaseListByTimeSingleRing(CrossIdAndTimeDTO crossIdAndTimeDTO) {
        List<SaveSchemeConfigDTO.PhaseListElement> phaseListByTime = new ArrayList<>();

        PhaseListByTimeVO phaseListByTimeVO = phaseListByTime(crossIdAndTimeDTO);
        SaveSchemeConfigDTO.PhaseScheme phaseScheme = phaseListByTimeVO.getPhaseScheme();
        if (phaseScheme != null) {
            List<SaveSchemeConfigDTO.RingListElement> ringList = phaseScheme.getRingList();
            SaveSchemeConfigDTO.RingListElement ringListElement = ringList.get(0);
            List<SaveSchemeConfigDTO.GroupListElement> groupList = ringListElement.getGroupList();

            for (SaveSchemeConfigDTO.GroupListElement groupListElement : groupList) {
                List<SaveSchemeConfigDTO.PhaseListElement> phaseList = groupListElement.getPhaseList();
                phaseListByTime.addAll(phaseList);
            }
        }

        return phaseListByTime;
    }

    private PhaseListByTimeVO getPhaseListByTimeVO(String planName, Date datetime, RunningPlanDTO runningPlanDTO,
                                                   CrossIdDTO crossIdDTO) {
        PhaseListByTimeVO phaseListByTimeVO = new PhaseListByTimeVO();
        List<RunningPlanDTO.TimeListElement> timeList = new ArrayList<>();
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = runningPlanDTO.getDailyPlanList();
        for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
            String name = dailyPlan.getName();
            if (Objects.equals(planName, name)) {
                timeList = dailyPlan.getTimeList();
            }
        }
        String hourMinuteStr = hourMinuteSdf.format(datetime);
        String hourMinuteStrReplace = hourMinuteStr.replace(":", "");
        int hourMinute = Integer.parseInt(hourMinuteStrReplace);
        String schemeName = null;
        for (RunningPlanDTO.TimeListElement timeListElement : timeList) {
            String startTimeStr = timeListElement.getStartTime();
            String startTimeStrReplace = startTimeStr.replace(":", "");
            int startTime = Integer.parseInt(startTimeStrReplace);
            String endTimeStr = timeListElement.getEndTime();
            String endTimeStrReplace = endTimeStr.replace(":", "");
            int endTime = Integer.parseInt(endTimeStrReplace);
            if (hourMinute >= startTime && hourMinute <= endTime) {
                String[] startEndTime = timeListElement.getStartEndTime();
                startEndTime[0] = startTimeStr;
                startEndTime[1] = endTimeStr;
                phaseListByTimeVO.setStartEndTime(startEndTime);
                schemeName = timeListElement.getSchemeName();
            }
        }
        // 查询方案
        SaveSchemeConfigDTO saveSchemeConfigDTO = schemeConfigServiceImpl.listSchemeConfig(crossIdDTO);
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            String name = phaseScheme.getName();
            if (Objects.equals(name, schemeName)) {
                phaseListByTimeVO.setPhaseScheme(phaseScheme);
            }
        }
        return phaseListByTimeVO;
    }

    @NotNull
    private static ControlCommandVO getControlCommandVO(CrossIdDTO crossIdDTO) {
        ControlCommandVO commandVO = new ControlCommandVO();
        commandVO.setCrossCode(crossIdDTO.getCrossId());
        commandVO.setCommand(1); //控制标识：1是；0否
        commandVO.setDuration(0); //长期
        return commandVO;
    }

    private List<SchemeSendVO.Pattern> buildPatternList(String crossId, Integer schemeId) {
        List<SchemeSendVO.Pattern> patternList = new ArrayList<>();
        CrossSchemePO crossSchemePO = crossSchemeMapper.selectById(schemeId);
        SchemeSendVO.Pattern pattern = new SchemeSendVO.Pattern();
        pattern.setPatternNo(crossSchemePO.getSchemeNo());
        pattern.setPatternName(crossSchemePO.getName());
        pattern.setCycle(crossSchemePO.getCycle().toString());
        pattern.setCoordPhase(crossSchemePO.getCoordPhase().toString());
        pattern.setOffset(crossSchemePO.getOffset().toString());
        // 构造环列表
        List<SchemeSendVO.Pattern.Ring> rings = buildRings(crossId, schemeId);
        pattern.setRings(rings);
        patternList.add(pattern);

        return patternList;
    }

    private List<SchemeSendVO.Pattern.Ring> buildRings(String crossId, Integer schemeId) {
        List<SchemeSendVO.Pattern.Ring> rings = new ArrayList<>();
        // 根据路口ID和方案ID查询相位集合，根据ringNo分组，特殊控制ringNo为0
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        Map<Integer, List<CrossPhasePO>> collectMap = crossPhasePOList.stream()
                .collect(Collectors.groupingBy(CrossPhasePO::getRingNo));
        for (Map.Entry<Integer, List<CrossPhasePO>> entry : collectMap.entrySet()) {
            SchemeSendVO.Pattern.Ring ring = new SchemeSendVO.Pattern.Ring();
            ring.setRingNo(entry.getKey().toString());
            // 构造相位列表
            List<CrossPhasePO> phases = entry.getValue();
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = buildPhaseList(phases);
            ring.setPhaseList(phaseList);
            rings.add(ring);
        }
        return rings;
    }

    private List<SchemeSendVO.Pattern.Ring.Phase> buildPhaseList(List<CrossPhasePO> phases) {
        List<SchemeSendVO.Pattern.Ring.Phase> phaseList = new ArrayList<>();
        for (CrossPhasePO phasePO : phases) {
            SchemeSendVO.Pattern.Ring.Phase phase = new SchemeSendVO.Pattern.Ring.Phase();
            phase.setPhaseNo(phasePO.getPhaseNo());
            phase.setPhaseName(phasePO.getName());
            phase.setSort(phasePO.getSort().toString());
            phase.setGroupNo(phasePO.getGroupNo());
            phase.setControlMode(phasePO.getControlMode().toString());
            phase.setMinGreenTime(phasePO.getMinGreenTime().toString());
            phase.setMaxGreenTime(phasePO.getMaxGreenTime().toString());
            phase.setPhaseTime(phasePO.getPhaseTime().toString());
            phase.setGreenTime(phasePO.getGreenTime().toString());
            phase.setGreenFlashTime(phasePO.getGreenFlashTime().toString());
            phase.setPedFlashTime(phasePO.getPedFlashTime().toString());
            phase.setYellowTime(phasePO.getYellowTime().toString());
            phase.setRedTime(phasePO.getRedTime().toString());

            phaseList.add(phase);
        }
//        Collections.sort(phaseList, (p1, p2) -> p1.getGroupNo() - p2.getGroupNo() &&  (p3, p4) -> Integer.valueOf(p3.getSort()) - Integer.valueOf(p3.getSort()));
        List<SchemeSendVO.Pattern.Ring.Phase> sort = phaseList.stream().sorted(Comparator.comparingInt(SchemeSendVO.Pattern.Ring.Phase::getGroupNo).thenComparing(SchemeSendVO.Pattern.Ring.Phase::getSort)).collect(Collectors.toList());

        return sort;
    }

    private List<ScheduleSendVO.Schedule> buildSchedules(String crossId, ScheduleIdDTO scheduleIdDTO) {
        List<ScheduleSendVO.Schedule> schedules = new ArrayList<>();
        Integer scheduleId = scheduleIdDTO.getScheduleId();
        List<CrossSchedulesPlanPO> crossSchedulesPlanPOList =
                crossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, scheduleId);
        ScheduleSendVO.Schedule schedule = new ScheduleSendVO.Schedule();
        List<ScheduleSendVO.Schedule.Week> weeks = new ArrayList<>();
        List<ScheduleSendVO.Schedule.SpecialDay> specialDays = new ArrayList<>();
        for (CrossSchedulesPlanPO crossSchedulesPlanPO : crossSchedulesPlanPOList) {
            Integer planId = crossSchedulesPlanPO.getPlanId();
            Integer week = crossSchedulesPlanPO.getWeek();
            Date specialDate = crossSchedulesPlanPO.getSpecialDate();
            if (week != 0) {
                // 添加至星期列表
                addToWeeks(weeks, week, planId);
            } else {
                // 添加至特殊日期列表
                addToSpecialDays(specialDays, specialDate, planId);
            }
        }
        schedule.setWeeks(weeks);
        schedule.setSpecialDays(specialDays);
        CrossSchedulesPO crossSchedulesPO = crossSchedulesMapper.selectById(scheduleId);
        schedule.setScheduleNo(crossSchedulesPO.getScheduleNo());
        schedules.add(schedule);
        return schedules;
    }

    private void addToSpecialDays(List<ScheduleSendVO.Schedule.SpecialDay> specialDays,
                                  Date specialDate, Integer planId) {
        ScheduleSendVO.Schedule.SpecialDay specialDay = new ScheduleSendVO.Schedule.SpecialDay();
        specialDay.setDateStr(sdf.format(specialDate));
        String planNo = getPlanNo(planId);
        specialDay.setPlanNo(Integer.parseInt(planNo));
        specialDays.add(specialDay);
    }

    private void addToWeeks(List<ScheduleSendVO.Schedule.Week> weeks, Integer week, Integer planId) {
        ScheduleSendVO.Schedule.Week weekObj = new ScheduleSendVO.Schedule.Week();
        weekObj.setWeekNum(week);
        String planNo = getPlanNo(planId);
        weekObj.setPlanNo(Integer.parseInt(planNo));
        weeks.add(weekObj);
    }

    private String getPlanNo(Integer planId) {
        CrossPlanPO crossPlanPO = crossPlanMapper.selectById(planId);
        return crossPlanPO.getPlanNo();
    }
}
